@@ -2,7 +2,7 @@ import codecs
 import json
 import os
 from datetime import datetime
-from typing import List, Union
+from typing import List, Union, Set, Dict
 from urllib.parse import urljoin
 import requests
 from bs4 import BeautifulSoup
@@ -10,8 +10,13 @@ from dotenv import load_dotenv
 from ruamel.yaml import YAML
 
 from teko.helpers.clog import CLog
-from teko.helpers.exceptions import TekoJiraException
-from teko.models.jira_testcase import JiraTestcase
+from teko.helpers.exceptions import raise_jira_exception
+from teko.models.jira_testcase import JiraTestData
+
+
+def quoted_testcase_name(name: str):
+    name = name.replace("'", "\\'")
+    return name
 
 
 class JiraService:
@@ -43,15 +48,15 @@ class JiraService:
         confluence_password = config.get('confluence_password') or os.getenv('CONFLUENCE_PASSWORD')
 
         if not self.project_key or not server or not jira_username or not jira_password or not confluence_username or not confluence_password:
-            msg = "No valid JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, JIRA_USERNAME, " \
-                  "JIRA_PASSWORD, CONFLUENCE_USERNAME and CONFLUENCE_PASSWORD environment variables first."
-            CLog.error(msg)
-            raise TekoJiraException(msg)
+            raise_jira_exception("No valid JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, "
+                                 "JIRA_USERNAME, JIRA_PASSWORD, CONFLUENCE_USERNAME and CONFLUENCE_PASSWORD "
+                                 "environment variables first.")
 
         if not server.startswith("http://") and not server.startswith("https://"):
             server = "https://" + server
-        self.base_api_url = urljoin(server, '/rest/atm/1.0')
-        self.base_api_tests_url = urljoin(server, '/rest/tests/1.0')
+        self.base_api_url = urljoin(server, 'rest/atm/1.0/')
+        self.base_api_tests_url = urljoin(server, 'rest/tests/1.0/')
+        self.base_api_2_url = urljoin(server, 'rest/api/2/')
 
         self.jira_session = requests.session()
         self.jira_session.auth = (jira_username, jira_password)
@@ -59,46 +64,39 @@ class JiraService:
         self.confluence_session = requests.session()
         self.confluence_session.auth = (confluence_username, confluence_password)
 
-    def search_testcases(self, issues=[], name=None):
-        """
-
-        :param issues: list of issue keys to filter
-        :return:
-        """
+    def search_testcase(self, name) -> Union[JiraTestData, None]:
         query = f'projectKey = "{self.project_key}"'
-        if issues:
-            query += f' AND issueKeys IN ({",".join(issues)})'
         if name:
-            query += f' AND name = "{name}"'
+            query += f' AND name = "{quoted_testcase_name(name)}"'
 
         params = {
             "query": query
         }
 
-        url = self.base_api_url + "/testcase/search"
+        url = urljoin(self.base_api_url, "testcase/search")
 
         res = self.jira_session.get(url=url, params=params)
 
         if res.status_code != 200:
-            CLog.error(f"Failed to search for test case {name}.")
+            CLog.error(f"Failed to search for test case \"{name}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request GET to {url} with params {params}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
                 CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
-            return []
+            return None
+
         raw = res.text
 
         testcases_json = json.loads(raw)
 
-        testcases = []
-        for t in testcases_json:
-            testcases.append(JiraTestcase().from_dict(t))
-        return testcases
+        return JiraTestData().from_dict(testcases_json[0]) if testcases_json else None
 
     def delete_trace_link(self, link_id):
-        url = self.base_api_url + f"/tracelink/{link_id}"
+        url = urljoin(self.base_api_url, f"tracelink/{link_id}")
 
         res = self.jira_session.delete(url=url)
 
@@ -108,9 +106,12 @@ class JiraService:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request DELETE to {url}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
                 CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             return
+
         raw = res.text
 
         testcases_json = json.loads(raw)
@@ -118,36 +119,40 @@ class JiraService:
         CLog.info(f"{json.dumps(testcases_json)}")
 
     def empty_trace_links(self, test_id):
-        url_confluence = self.base_api_tests_url + f"/testcase/{test_id}/tracelinks/confluencepage"
+        url_confluence = urljoin(self.base_api_tests_url, f"testcase/{test_id}/tracelinks/confluencepage")
 
         res = self.jira_session.get(url=url_confluence)
 
         if res.status_code != 200:
-            CLog.error(f"Failed to clear confluence links for test case {test_id}.")
+            CLog.error(f"Failed to clear confluence links of test case \"{test_id}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request GET to {url_confluence}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
         else:
             raw = res.text
             links = json.loads(raw)
             for link in links:
                 self.delete_trace_link(link["id"])
 
-        url_web = self.base_api_tests_url + f"/testcase/{test_id}/tracelinks/weblink"
+        url_web = urljoin(self.base_api_tests_url, f"testcase/{test_id}/tracelinks/weblink")
 
         res = self.jira_session.get(url=url_web)
 
         if res.status_code != 200:
-            CLog.error(f"Failed to clear web links for test case {test_id}.")
+            CLog.error(f"Failed to clear web links of test case \"{test_id}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request GET to {url_web}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
         else:
             raw = res.text
             links = json.loads(raw)
@@ -155,7 +160,7 @@ class JiraService:
                 self.delete_trace_link(link["id"])
 
     def add_confluence_links(self, test_id, confluences: List[str]):
-        url = self.base_api_tests_url + "/tracelink/bulk/create"
+        url = urljoin(self.base_api_tests_url, "tracelink/bulk/create")
 
         if not confluences:
             return
@@ -165,13 +170,15 @@ class JiraService:
         for confluence_url in confluences:
             res = self.confluence_session.get(url=confluence_url)
             if res.status_code != 200:
-                CLog.error(f"Failed to get id of confluence page {confluence_url}.")
+                CLog.error(f"Failed to get the id of confluence page {confluence_url}.")
                 if res.status_code == 401:
                     CLog.error(f"Unauthorized: Authenticated failed.")
                 elif res.status_code == 403:
                     CLog.error(f"Forbidden: You don't have permission to send request GET to {confluence_url}.")
+                elif not res.text:
+                    CLog.error(f"HTTP Error {res.status_code}. No response message.")
                 else:
-                    CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
+                    CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
                 self.success = False
                 return
 
@@ -188,17 +195,19 @@ class JiraService:
         res = self.jira_session.post(url=url, json=confluences_data)
 
         if res.status_code != 200:
-            CLog.error(f"Failed to add confluence links for test case {test_id}.")
+            CLog.error(f"Failed to add confluence links for test case \"{test_id}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {confluences_data}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
                 CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
 
     def add_web_links(self, test_id, webs: List[Union[str, dict]]):
-        url = self.base_api_tests_url + "/tracelink/bulk/create"
+        url = urljoin(self.base_api_tests_url, "tracelink/bulk/create")
 
         if not webs:
             return
@@ -224,11 +233,13 @@ class JiraService:
         res = self.jira_session.post(url=url, json=webs_data)
 
         if res.status_code != 200:
-            CLog.error(f"Failed to add web links for test case {test_id}.")
+            CLog.error(f"Failed to add web links for test case \"{test_id}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {webs_data}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
                 CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
@@ -247,7 +258,7 @@ class JiraService:
             "type": type
         }
 
-        url = self.base_api_url + "/folder"
+        url = urljoin(self.base_api_url, "folder")
 
         """
         Ignore failure
@@ -260,60 +271,97 @@ class JiraService:
             CLog.error(f"Cannot create {type} folder {name}.")
             CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
         if res.status_code != 201:
-            CLog.info(f"{type} Folder {name} has already exists.")
+            CLog.info(f"{type} folder {name} has already exists.")
 
-    def get_test_id(self, test_key):
-        url = self.base_api_tests_url + f"/testcase/{test_key}"
+    def get_test_id(self, test_key) -> Union[int, None]:
+        url = urljoin(self.base_api_tests_url, f"testcase/{test_key}")
 
         res = self.jira_session.get(url=url, params={"fields": "id"})
         if res.status_code != 200:
-            CLog.error(f"Failed to get id of test case {test_key}.")
+            CLog.error(f"Failed to get the id of test case {test_key}.")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request GET to {url}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
             self.success = False
             return
 
         raw = res.text
         return json.loads(raw).get('id')
 
-    def create_or_update_testcases_by_name(self, testcases: List[JiraTestcase], issues=[]):
-        CLog.info("Creating/Updating test cases.")
-        self.success = True
+    def get_cycle_id(self, cycle_key) -> Union[int, None]:
+        url = urljoin(self.base_api_tests_url, f"testrun/{cycle_key}")
 
-        folders = set()
-
-        for testcase in testcases:
-            if testcase.folder:
-                folders.add(testcase.folder)
-
-        for folder in folders:
-            self.create_folder(folder, "TEST_CASE")
-
-        for testcase in testcases:
-            existing_testcases = self.search_testcases(issues=issues, name=testcase.name)
-            if not existing_testcases:
-                key = self.create_testcase(testcase)
-                testcase.key = key
+        res = self.jira_session.get(url=url, params={"fields": "id"})
+        if res.status_code != 200:
+            CLog.error(f"Failed to get the id of test case {cycle_key}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
-                key = self.update_testcase(existing_testcases[0].key, testcase)
-                testcase.key = key
-            if key:
-                self.add_links(
-                    test_id=self.get_test_id(key),
-                    confluences=testcase.confluenceLinks,
-                    webs=testcase.webLinks
-                )
-        if not self.success:
-            msg = "Creating/updating test cases process does not succeeded."
-            CLog.info(msg)
-            raise TekoJiraException(msg)
-        return testcases
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
+            self.success = False
+            return
 
-    def prepare_testcase_data(self, testcase: JiraTestcase):
+        raw = res.text
+        return json.loads(raw).get('id')
+
+    def get_issue_id(self, issue_key) -> Union[str, None]:
+        url = urljoin(self.base_api_2_url, f"issue/{issue_key}")
+        res = self.jira_session.get(url=url, params={"fields": "id"})
+        if res.status_code != 200:
+            CLog.error(f"Failed to get the id of issue {issue_key}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
+            else:
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
+            self.success = False
+            return
+
+        raw = res.text
+        return json.loads(raw).get('id')
+
+    def add_cycle_to_issues(self, cycle_key: str, issue_keys: Set[str], issue_ids: Dict):
+        url = urljoin(self.base_api_tests_url, "tracelink/issue/testrun/bulk/create")
+
+        cycle_id = self.get_cycle_id(cycle_key)
+
+        data = [
+            {
+                "issueId": issue_ids[issue_key],
+                "testRunId": cycle_id,
+                "typeId": 2
+            } for issue_key in issue_keys
+        ]
+
+        res = self.jira_session.post(url=url, json=data)
+        if res.status_code != 200:
+            CLog.error(f"Failed to get add cycle {cycle_key} into issue {', '.join(sorted(issue_keys))}.")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request GET to {url}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
+            else:
+                CLog.error(f"HTTP Error {res.status_code}. View detail at: {res.url}.")
+            self.success = False
+            return
+
+        CLog.info(f"Added cycle {cycle_key} into issue {', '.join(issue_keys)} successfully.")
+
+    def prepare_testcase_data(self, testcase: JiraTestData) -> dict:
         if not testcase.projectKey:
             testcase.projectKey = self.project_key
 
@@ -323,7 +371,8 @@ class JiraService:
             'folder': testcase.folder,
             'objective': testcase.objective,
             'precondition': testcase.precondition,
-            'status': testcase.status
+            'status': testcase.status,
+            'labels': testcase.labels
         }
 
         data.pop("status")
@@ -345,50 +394,87 @@ class JiraService:
 
         return data
 
-    def update_testcase(self, key: str, testcase: JiraTestcase):
-        url = self.base_api_url + f"/testcase/{key}"
+    def create_testcase(self, testcase: JiraTestData) -> Union[str, None]:
         data = self.prepare_testcase_data(testcase)
-        data.pop('projectKey')
-        res = self.jira_session.put(url=url, json=data)
-
-        if res.status_code != 200:
-            CLog.error(f"Failed to update test case `{testcase.name}`.")
-            if res.status_code == 401:
-                CLog.error(f"Unauthorized: Authenticated failed.")
-            elif res.status_code == 403:
-                CLog.error(f"Forbidden: You don't have permission to send request PUT to {url} with payload {data}.")
-            else:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
-            self.success = False
-            return None
-
-        CLog.info(f"Updated test `{testcase.name}` successfully. Test key: {key}.")
-        return key
-
-    def create_testcase(self, testcase: JiraTestcase):
-        data = self.prepare_testcase_data(testcase)
-        url = self.base_api_url + "/testcase"
+        url = urljoin(self.base_api_url, "testcase")
         res = self.jira_session.post(url=url, json=data)
 
         if res.status_code != 201:
-            CLog.error(f"Failed to create test case `{testcase.name}`.")
+            CLog.error(f"Failed to create test case \"{testcase.name}\".")
             if res.status_code == 401:
                 CLog.error(f"Unauthorized: Authenticated failed.")
             elif res.status_code == 403:
                 CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
             else:
                 CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
             self.success = False
-            return None
+            return
 
         raw = res.text
 
         testcase_json = json.loads(raw)
         key = testcase_json['key']
-        CLog.info(f"Created test `{testcase.name}` successfully. Test key: {key}.")
+        CLog.info(f"Created test \"{testcase.name}\" successfully. Test key: {key}.")
         return key
 
-    def prepare_testrun_data(self, testcase: JiraTestcase):
+    def update_testcase(self, key: str, testcase: JiraTestData) -> Union[str, None]:
+        url = urljoin(self.base_api_url, f"testcase/{key}")
+        data = self.prepare_testcase_data(testcase)
+        data.pop('projectKey')
+        res = self.jira_session.put(url=url, json=data)
+
+        if res.status_code != 200:
+            CLog.error(f"Failed to update test case \"{testcase.name}\".")
+            if res.status_code == 401:
+                CLog.error(f"Unauthorized: Authenticated failed.")
+            elif res.status_code == 403:
+                CLog.error(f"Forbidden: You don't have permission to send request PUT to {url} with payload {data}.")
+            elif not res.text:
+                CLog.error(f"HTTP Error {res.status_code}. No response message.")
+            else:
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
+            self.success = False
+            return
+
+        CLog.info(f"Updated test \"{testcase.name}\" successfully. Test key: {key}.")
+        return key
+
+    def create_or_update_testcases_by_name(self, testcases: List[JiraTestData]) -> List[JiraTestData]:
+        CLog.info("Creating/Updating test cases.")
+        self.success = True
+
+        folders = set()
+
+        for testcase in testcases:
+            if testcase.folder:
+                folders.add(testcase.folder)
+
+        for folder in folders:
+            self.create_folder(folder, "TEST_CASE")
+
+        for testcase in testcases:
+            existing_testcase = self.search_testcase(name=testcase.name)
+            if not existing_testcase:
+                key = self.create_testcase(testcase)
+                testcase.key = key
+            else:
+                key = self.update_testcase(existing_testcase.key, testcase)
+                testcase.key = key
+            if key:
+                self.add_links(
+                    test_id=self.get_test_id(key),
+                    confluences=testcase.confluenceLinks,
+                    webs=testcase.webLinks
+                )
+
+        if not self.success:
+            raise_jira_exception("Creating/updating test cases process did not succeed.")
+
+        return testcases
+
+    def prepare_testrun_data(self, testcase: JiraTestData) -> dict:
         if not testcase.projectKey:
             testcase.projectKey = self.project_key
 
@@ -402,38 +488,50 @@ class JiraService:
 
         return data
 
-    def create_test_cycle(self, testcases: List[JiraTestcase]):
+    def create_test_cycles(self, testcases: List[JiraTestData]):
         CLog.info("Creating test cycles.")
         self.success = True
-        items_by_folder = dict()
+
+        cycles_by_folder = dict()
+
+        issue_ids = dict()
+
+        issues_by_folder = dict()
+
         for testcase in testcases:
-            existing_testcases = self.search_testcases(name=testcase.name)
-            if not existing_testcases:
-                CLog.error(f"Testcase with name `{testcase.name}` not existed.")
+            existing_testcase = self.search_testcase(name=testcase.name)
+            if not existing_testcase:
+                CLog.error(f"Testcase with name \"{testcase.name}\" not existed.")
                 self.success = False
             else:
-                testcase.key = existing_testcases[0].key
-                CLog.info(f"Testcase with name `{testcase.name}` existed.")
-                if items_by_folder.get(testcase.testrun_folder) is None:
-                    items_by_folder[testcase.testrun_folder] = []
-                items_by_folder[testcase.testrun_folder].append(self.prepare_testrun_data(testcase))
+                testcase.key = existing_testcase.key
+                CLog.info(f"Testcase with name \"{testcase.name}\" existed.")
+                folder = testcase.testrun_folder
+                if cycles_by_folder.get(folder) is None:
+                    cycles_by_folder[folder] = []
+                if issues_by_folder.get(folder) is None:
+                    issues_by_folder[folder] = set()
+
+                cycles_by_folder[folder].append(self.prepare_testrun_data(testcase))
+                for issue_key in existing_testcase.issueLinks:
+                    if not issue_ids.get(issue_key):
+                        issue_ids[issue_key] = self.get_issue_id(issue_key)
+                    issues_by_folder[folder].add(issue_key)
 
         if not self.success:
-            msg = "Creating test cycle process does not succeeded."
-            CLog.error(msg)
-            raise TekoJiraException(msg)
+            raise_jira_exception("Creating test cycle process did not succeed.")
 
-        url = self.base_api_url + "/testrun"
+        url = urljoin(self.base_api_url, "testrun")
 
-        for folder in items_by_folder.keys():
+        for folder in cycles_by_folder.keys():
             self.create_folder(folder, "TEST_RUN")
 
-        for folder, items in items_by_folder.items():
+        for folder, cycles in cycles_by_folder.items():
             data = {
                 "projectKey": self.project_key,
                 "name": f"[{self.project_key}] Cycle {datetime.now()}",
                 "folder": folder,
-                "items": items
+                "items": cycles
             }
 
             if not folder:
@@ -447,6 +545,8 @@ class JiraService:
                     CLog.error(f"Unauthorized: Authenticated failed.")
                 elif res.status_code == 403:
                     CLog.error(f"Forbidden: You don't have permission to send request POST to {url} with payload {data}.")
+                elif not res.text:
+                    CLog.error(f"HTTP Error {res.status_code}. No response message.")
                 else:
                     CLog.error(f"HTTP Error {res.status_code}: {res.text}.")
                 self.success = False
@@ -454,20 +554,20 @@ class JiraService:
 
             raw = res.text
 
-            testcase_json = json.loads(raw)
-            key = testcase_json['key']
+            testcycle_json = json.loads(raw)
+            key = testcycle_json['key']
             if folder:
                 CLog.info(f"Test cycle {key} created in folder {folder}.")
             else:
                 CLog.info(f"Test cycle {key} created.")
 
+            self.add_cycle_to_issues(key, issues_by_folder[folder], issue_ids)
+
         if not self.success:
-            msg = "Creating test cycle process does not succeeded."
-            CLog.error(msg)
-            raise TekoJiraException(msg)
-        
+            raise_jira_exception("Creating test cycles process does not succeeded.")
+
     @staticmethod
-    def read_testcases_from_file(testcase_file):
+    def read_test_from_file(testcase_file):
         """
 
         :param testcase_file: yaml or json file
@@ -480,9 +580,7 @@ class JiraService:
                 try:
                     testcase_data = json.load(f)
                 except Exception:
-                    msg = "Invalid json format."
-                    CLog.error(msg)
-                    raise TekoJiraException(msg) from None
+                    raise_jira_exception("Invalid json format.")
 
         elif ext == ".yaml":
             with codecs.open(testcase_file, "r", encoding="utf-8") as f:
@@ -490,31 +588,30 @@ class JiraService:
                 try:
                     testcase_data = yaml.load(f)
                 except Exception:
-                    msg = "Invalid yaml format"
-                    CLog.error(msg)
-                    raise TekoJiraException(msg) from None
+                    raise_jira_exception("Invalid yaml format.")
         else:
-            msg = f"File format not supported: {ext}."
-            CLog.error(msg)
-            raise TekoJiraException(msg)
+            raise_jira_exception(f"File format not supported: {ext}.")
 
         testcases = []
         for t in testcase_data:
+            name = t.get('name')
+            if not name:
+                raise_jira_exception("A test case is missing its name.")
+            if name.count('\\') or name.count('"'):
+                raise_jira_exception("Invalid character(s) in the testcase name. Avoid using '\\' and '\"'.")
             if t.get('folder') and t['folder'][0] != '/':
                 t['folder'] = '/' + t['folder']
             if t.get('testrun_folder') and t['testrun_folder'][0] != '/':
                 t['testrun_folder'] = '/' + t['testrun_folder']
             try:
-                testcases.append(JiraTestcase().from_dict(t))
+                testcases.append(JiraTestData().from_dict(t))
             except Exception:
-                msg = f"Failed to parse test data from file {testcase_file}."
-                CLog.error(msg)
-                raise TekoJiraException(msg) from None
+                raise_jira_exception(f"Failed to parse test data from file {testcase_file}.")
 
         return testcases
 
     @staticmethod
-    def write_testcases_to_file(testcase_file, testcases):
+    def write_test_to_file(testcase_file, testcases):
         """
 
         :param testcase_file: yaml or json file
@@ -527,7 +624,7 @@ class JiraService:
             testcases_json.append(data)
 
         filename, ext = os.path.splitext(testcase_file)
-        CLog.info(filename, ext)
+        CLog.info(f"Writing {os.path.abspath(testcase_file)}.")
         if ext == ".json":
             with codecs.open(testcase_file, "w", encoding="utf-8") as f:
                 json.dump(testcases_json, f, indent=2)
@@ -537,21 +634,19 @@ class JiraService:
                 yaml.indent(offset=2)
                 yaml.dump(testcases_json, f)
         else:
-            msg = f"File format not supported: {ext}."
-            CLog.error(msg)
-            raise TekoJiraException(msg) from None
+            raise_jira_exception(f"File format not supported: {ext}.")
 
 
 if __name__ == "__main__":
     jira_srv = JiraService()
 
-    testcase_file = "../../../sample/testcases.json"
-    testcases = JiraService.read_testcases_from_file(testcase_file)
+    testcase_file = "../../../sample/testcases.yaml"
+    testcases = JiraService.read_test_from_file(testcase_file)
     jira_srv.create_or_update_testcases_by_name(testcases)
 
-    testcycle_file = "../../../sample/testcycles.json"
-    testcases = JiraService.read_testcases_from_file(testcycle_file)
-    jira_srv.create_test_cycle(testcases)
+    testcycle_file = "../../../sample/testcycles.yaml"
+    testcases = JiraService.read_test_from_file(testcycle_file)
+    jira_srv.create_test_cycles(testcases)
 
 
 
